# -*- coding: utf-8 -*-

# Dell OpenManage Ansible Modules
# Version 9.12.0
# Copyright (C) 2025 Dell Inc. or its subsidiaries. All Rights Reserved.

# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:

#    * Redistributions of source code must retain the above copyright notice,
#      this list of conditions and the following disclaimer.

#    * Redistributions in binary form must reproduce the above copyright notice,
#      this list of conditions and the following disclaimer in the documentation
#      and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#


GET_IDRAC_CHASSIS_URI = "/redfish/v1/Chassis"
NA = "Not Available"


class IDRACEnclosureInfo(object):
    def __init__(self, idrac):
        self.idrac = idrac

    def get_enclosure_data(self, resp):
        dellchasis = resp.get("Oem", {}).get("Dell", {}).get("DellChassisEnclosure", {})
        output = {
            "AssetTag": NA if (asset := resp.get("AssetTag")) == "" else asset,
            "Connector": str(dellchasis.get("Connector")),
            "DeviceDescription": resp.get("Description"),
            "EMMCount": str(dellchasis.get("Links").get("DellEnclosureEMMCollection@odata.count")),
            "FQDD": resp.get("Id", NA),
            "FanCount": NA,
            "Key": resp.get("Id", NA),
            "PSUCount": NA,
            "PrimaryStatus": resp.get("Status", {}).get("Health", NA),
            "ProductName": resp.get("Name", NA),
            "ServiceTag": NA if (svctag := dellchasis.get("ServiceTag")) is None else svctag,
            "SlotCount": str(dellchasis.get("SlotCount", NA)),
            "State": NA,
            "Version": dellchasis.get("Version", NA),
            "WiredOrder": str(dellchasis.get("WiredOrder", NA))
        }
        return output

    def get_enclosure_system_info(self):
        output = []
        resp = self.idrac.invoke_request(method='GET', uri=GET_IDRAC_CHASSIS_URI)
        for each_member in resp.json_data.get("Members", []):
            if 'Enclosure' in each_member['@odata.id']:
                enc_resp = self.idrac.invoke_request(method='GET', uri=each_member['@odata.id'])
                output.append(self.get_enclosure_data(enc_resp.json_data))
        return output

    def get_controller_enclosure_sensor_info(self, enclosure_system_info):
        output = [{"FQDD": each.get("FQDD"), "Key": each.get("Key")} for each in enclosure_system_info]
        return output
