#!/usr/bin/python
from __future__ import absolute_import, division, print_function

# Copyright: (c) 2022 Fortinet
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

__metaclass__ = type

ANSIBLE_METADATA = {
    "status": ["preview"],
    "supported_by": "community",
    "metadata_version": "1.1",
}

DOCUMENTATION = """
---
module: fortios_system_vdom_exception
short_description: Global configuration objects that can be configured independently across different ha peers for all VDOMs or for the defined VDOM scope in
   Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS (FOS) device by allowing the
      user to set and modify system feature and vdom_exception category.
      Examples include all parameters and values need to be adjusted to datasources before usage.
      Tested with FOS v6.0.0
version_added: "2.0.0"
author:
    - Link Zheng (@chillancezen)
    - Jie Xue (@JieX19)
    - Hongbin Lu (@fgtdev-hblu)
    - Frank Shen (@frankshen01)
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - We highly recommend using your own value as the id instead of 0, while '0' is a special placeholder that allows the backend to assign the latest
       available number for the object, it does have limitations. Please find more details in Q&A.
    - Legacy fortiosapi has been deprecated, httpapi is the preferred way to run playbooks

    - The module supports check_mode.

requirements:
    - ansible>=2.15
options:
    access_token:
        description:
            - Token-based authentication.
              Generated from GUI of Fortigate.
        type: str
        required: false
    enable_log:
        description:
            - Enable/Disable logging for task.
        type: bool
        required: false
        default: false
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        type: str
        default: root
    member_path:
        type: str
        description:
            - Member attribute path to operate on.
            - Delimited by a slash character if there are more than one attribute.
            - Parameter marked with member_path is legitimate for doing member operation.
    member_state:
        type: str
        description:
            - Add or delete a member under specified attribute path.
            - When member_state is specified, the state option is ignored.
        choices:
            - 'present'
            - 'absent'

    state:
        description:
            - Indicates whether to create or remove the object.
        type: str
        required: true
        choices:
            - 'present'
            - 'absent'
    system_vdom_exception:
        description:
            - Global configuration objects that can be configured independently across different ha peers for all VDOMs or for the defined VDOM scope.
        default: null
        type: dict
        suboptions:
            id:
                description:
                    - Index (1 - 4096). see <a href='#notes'>Notes</a>.
                required: true
                type: int
            object:
                description:
                    - Name of the configuration object that can be configured independently for all VDOMs.
                type: str
                choices:
                    - 'log.fortianalyzer.setting'
                    - 'log.fortianalyzer.override-setting'
                    - 'log.fortianalyzer2.setting'
                    - 'log.fortianalyzer2.override-setting'
                    - 'log.fortianalyzer3.setting'
                    - 'log.fortianalyzer3.override-setting'
                    - 'log.fortianalyzer-cloud.setting'
                    - 'log.fortianalyzer-cloud.override-setting'
                    - 'log.syslogd.setting'
                    - 'log.syslogd.override-setting'
                    - 'log.syslogd2.setting'
                    - 'log.syslogd2.override-setting'
                    - 'log.syslogd3.setting'
                    - 'log.syslogd3.override-setting'
                    - 'log.syslogd4.setting'
                    - 'log.syslogd4.override-setting'
                    - 'system.gre-tunnel'
                    - 'system.central-management'
                    - 'system.csf'
                    - 'user.radius'
                    - 'log.syslogd.setting'
                    - 'log.syslogd.override-setting'
                    - 'firewall.address'
                    - 'system.interface'
                    - 'vpn.ipsec.phase1-interface'
                    - 'vpn.ipsec.phase2-interface'
                    - 'router.bgp'
                    - 'router.route-map'
                    - 'router.prefix-list'
                    - 'firewall.ippool'
                    - 'firewall.ippool6'
                    - 'router.static'
                    - 'router.static6'
                    - 'firewall.vip'
                    - 'firewall.vip6'
                    - 'system.sdwan'
                    - 'system.saml'
                    - 'router.policy'
                    - 'router.policy6'
                    - 'firewall.vip46'
                    - 'firewall.vip64'
            oid:
                description:
                    - Object ID.
                type: int
            scope:
                description:
                    - Determine whether the configuration object can be configured separately for all VDOMs or if some VDOMs share the same configuration.
                type: str
                choices:
                    - 'all'
                    - 'inclusive'
                    - 'exclusive'
            vdom:
                description:
                    - Names of the VDOMs.
                type: list
                elements: dict
                suboptions:
                    name:
                        description:
                            - VDOM name. Source system.vdom.name.
                        required: true
                        type: str
"""

EXAMPLES = """
- name: Global configuration objects that can be configured independently across different ha peers for all VDOMs or for the defined VDOM scope.
  fortinet.fortios.fortios_system_vdom_exception:
      vdom: "{{ vdom }}"
      state: "present"
      access_token: "<your_own_value>"
      system_vdom_exception:
          id: "3"
          object: "log.fortianalyzer.setting"
          oid: "32767"
          scope: "all"
          vdom:
              -
                  name: "default_name_8 (source system.vdom.name)"
"""

RETURN = """
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"
"""
from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.connection import Connection
from ansible_collections.fortinet.fortios.plugins.module_utils.fortios.fortios import (
    FortiOSHandler,
)
from ansible_collections.fortinet.fortios.plugins.module_utils.fortios.fortios import (
    check_legacy_fortiosapi,
)
from ansible_collections.fortinet.fortios.plugins.module_utils.fortios.fortios import (
    schema_to_module_spec,
)
from ansible_collections.fortinet.fortios.plugins.module_utils.fortios.fortios import (
    check_schema_versioning,
)
from ansible_collections.fortinet.fortios.plugins.module_utils.fortimanager.common import (
    FAIL_SOCKET_MSG,
)
from ansible_collections.fortinet.fortios.plugins.module_utils.fortios.data_post_processor import (
    remove_invalid_fields,
)
from ansible_collections.fortinet.fortios.plugins.module_utils.fortios.comparison import (
    is_same_comparison,
)
from ansible_collections.fortinet.fortios.plugins.module_utils.fortios.comparison import (
    serialize,
)
from ansible_collections.fortinet.fortios.plugins.module_utils.fortios.comparison import (
    find_current_values,
)
from ansible_collections.fortinet.fortios.plugins.module_utils.fortios.comparison import (
    unify_data_format,
)


def filter_system_vdom_exception_data(json):
    option_list = ["id", "object", "oid", "scope", "vdom"]

    json = remove_invalid_fields(json)
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def underscore_to_hyphen(data):
    new_data = None
    if isinstance(data, list):
        new_data = []
        for i, elem in enumerate(data):
            new_data.append(underscore_to_hyphen(elem))
    elif isinstance(data, dict):
        new_data = {}
        for k, v in data.items():
            new_data[k.replace("_", "-")] = underscore_to_hyphen(v)
    else:
        return data
    return new_data


def system_vdom_exception(data, fos, check_mode=False):

    state = None
    vdom = data["vdom"]
    state = data.get("state", None)
    system_vdom_exception_data = data["system_vdom_exception"]

    filtered_data = filter_system_vdom_exception_data(system_vdom_exception_data)
    converted_data = underscore_to_hyphen(filtered_data)

    # check_mode starts from here
    if check_mode:
        diff = {
            "before": "",
            "after": filtered_data,
        }
        mkeyname = fos.get_mkeyname(None, None)
        mkey = fos.get_mkey("system", "vdom-exception", filtered_data, vdom=vdom)
        current_data = fos.get("system", "vdom-exception", vdom=vdom, mkey=mkey)
        is_existed = (
            current_data
            and current_data.get("http_status") == 200
            and (
                mkeyname
                and isinstance(current_data.get("results"), list)
                and len(current_data["results"]) > 0
                or not mkeyname
                and current_data["results"]  # global object response
            )
        )

        # 2. if it exists and the state is 'present' then compare current settings with desired
        if state == "present" or state is True or state is None:
            # for non global modules, mkeyname must exist and it's a new module when mkey is None
            if mkeyname is not None and mkey is None:
                return False, True, filtered_data, diff

            # if mkey exists then compare each other
            # record exits and they're matched or not
            copied_filtered_data = filtered_data.copy()
            copied_filtered_data.pop(mkeyname, None)
            unified_filtered_data = unify_data_format(copied_filtered_data)

            current_data_results = current_data.get("results", {})
            current_config = (
                current_data_results[0]
                if mkeyname
                and isinstance(current_data_results, list)
                and len(current_data_results) > 0
                else current_data_results
            )
            if is_existed:
                unified_current_values = find_current_values(
                    unified_filtered_data,
                    unify_data_format(current_config),
                )

                is_same = is_same_comparison(
                    serialize(unified_current_values), serialize(unified_filtered_data)
                )

                return (
                    False,
                    not is_same,
                    filtered_data,
                    {"before": unified_current_values, "after": unified_filtered_data},
                )

            # record does not exist
            return False, True, filtered_data, diff

        if state == "absent":
            if mkey is None:
                return (
                    False,
                    False,
                    filtered_data,
                    {"before": current_data["results"][0], "after": ""},
                )

            if is_existed:
                return (
                    False,
                    True,
                    filtered_data,
                    {"before": current_data["results"][0], "after": ""},
                )
            return False, False, filtered_data, {}

        return True, False, {"reason: ": "Must provide state parameter"}, {}
    # pass post processed data to member operations
    # no need to do underscore_to_hyphen since do_member_operation handles it by itself
    data_copy = data.copy()
    data_copy["system_vdom_exception"] = filtered_data
    fos.do_member_operation(
        "system",
        "vdom-exception",
        data_copy,
    )

    if state == "present" or state is True:
        return fos.set("system", "vdom-exception", data=converted_data, vdom=vdom)

    elif state == "absent":
        return fos.delete(
            "system", "vdom-exception", mkey=converted_data["id"], vdom=vdom
        )
    else:
        fos._module.fail_json(msg="state must be present or absent!")


def is_successful_status(resp):
    return (
        "status" in resp
        and resp["status"] == "success"
        or "http_status" in resp
        and resp["http_status"] == 200
        or "http_method" in resp
        and resp["http_method"] == "DELETE"
        and resp["http_status"] == 404
    )


def fortios_system(data, fos, check_mode):

    if data["system_vdom_exception"]:
        resp = system_vdom_exception(data, fos, check_mode)
    else:
        fos._module.fail_json(msg="missing task body: %s" % ("system_vdom_exception"))
    if isinstance(resp, tuple) and len(resp) == 4:
        return resp
    return (
        not is_successful_status(resp),
        is_successful_status(resp)
        and (resp["revision_changed"] if "revision_changed" in resp else True),
        resp,
        {},
    )


versioned_schema = {
    "type": "list",
    "elements": "dict",
    "children": {
        "id": {"v_range": [["v6.0.0", ""]], "type": "integer", "required": True},
        "object": {
            "v_range": [["v6.0.0", ""]],
            "type": "string",
            "options": [
                {"value": "log.fortianalyzer.setting"},
                {"value": "log.fortianalyzer.override-setting"},
                {
                    "value": "log.fortianalyzer2.setting",
                    "v_range": [["v6.0.0", "v6.0.0"], ["v6.0.11", ""]],
                },
                {
                    "value": "log.fortianalyzer2.override-setting",
                    "v_range": [["v6.0.0", "v6.0.0"], ["v6.0.11", ""]],
                },
                {
                    "value": "log.fortianalyzer3.setting",
                    "v_range": [["v6.0.0", "v6.0.0"], ["v6.0.11", ""]],
                },
                {
                    "value": "log.fortianalyzer3.override-setting",
                    "v_range": [["v6.0.0", "v6.0.0"], ["v6.0.11", ""]],
                },
                {
                    "value": "log.fortianalyzer-cloud.setting",
                    "v_range": [["v6.2.0", ""]],
                },
                {
                    "value": "log.fortianalyzer-cloud.override-setting",
                    "v_range": [["v6.2.0", ""]],
                },
                {"value": "log.syslogd.setting", "v_range": [["v6.4.0", ""]]},
                {"value": "log.syslogd.override-setting", "v_range": [["v6.4.0", ""]]},
                {"value": "log.syslogd2.setting", "v_range": [["v6.4.0", ""]]},
                {"value": "log.syslogd2.override-setting", "v_range": [["v6.4.0", ""]]},
                {"value": "log.syslogd3.setting", "v_range": [["v6.4.0", ""]]},
                {"value": "log.syslogd3.override-setting", "v_range": [["v6.4.0", ""]]},
                {"value": "log.syslogd4.setting", "v_range": [["v6.4.0", ""]]},
                {"value": "log.syslogd4.override-setting", "v_range": [["v6.4.0", ""]]},
                {"value": "system.gre-tunnel", "v_range": [["v7.0.1", ""]]},
                {
                    "value": "system.central-management",
                    "v_range": [["v6.0.0", "v6.0.0"], ["v6.0.11", ""]],
                },
                {
                    "value": "system.csf",
                    "v_range": [["v6.0.0", "v6.0.0"], ["v6.0.11", ""]],
                },
                {
                    "value": "user.radius",
                    "v_range": [["v6.0.0", "v6.0.0"], ["v6.0.11", ""]],
                },
                {"value": "log.syslogd.setting", "v_range": [["v7.6.2", ""]]},
                {"value": "log.syslogd.override-setting", "v_range": [["v7.6.2", ""]]},
                {"value": "firewall.address", "v_range": [["v7.4.2", ""]]},
                {"value": "system.interface", "v_range": []},
                {"value": "vpn.ipsec.phase1-interface", "v_range": []},
                {"value": "vpn.ipsec.phase2-interface", "v_range": []},
                {"value": "router.bgp", "v_range": []},
                {"value": "router.route-map", "v_range": []},
                {"value": "router.prefix-list", "v_range": []},
                {"value": "firewall.ippool", "v_range": []},
                {"value": "firewall.ippool6", "v_range": []},
                {"value": "router.static", "v_range": []},
                {"value": "router.static6", "v_range": []},
                {"value": "firewall.vip", "v_range": []},
                {"value": "firewall.vip6", "v_range": []},
                {"value": "system.sdwan", "v_range": []},
                {"value": "system.saml", "v_range": []},
                {"value": "router.policy", "v_range": []},
                {"value": "router.policy6", "v_range": []},
                {"value": "firewall.vip46", "v_range": []},
                {"value": "firewall.vip64", "v_range": []},
            ],
        },
        "scope": {
            "v_range": [["v6.0.0", ""]],
            "type": "string",
            "options": [
                {"value": "all"},
                {"value": "inclusive"},
                {"value": "exclusive"},
            ],
        },
        "vdom": {
            "type": "list",
            "elements": "dict",
            "children": {
                "name": {
                    "v_range": [["v6.0.0", ""]],
                    "type": "string",
                    "required": True,
                }
            },
            "v_range": [["v6.0.0", ""]],
        },
        "oid": {
            "v_range": [["v6.0.0", "v6.0.11"], ["v6.2.3", "v6.2.3"]],
            "type": "integer",
        },
    },
    "v_range": [["v6.0.0", ""]],
}


def main():
    module_spec = schema_to_module_spec(versioned_schema)
    mkeyname = "id"
    fields = {
        "access_token": {"required": False, "type": "str", "no_log": True},
        "enable_log": {"required": False, "type": "bool", "default": False},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "member_path": {"required": False, "type": "str"},
        "member_state": {
            "type": "str",
            "required": False,
            "choices": ["present", "absent"],
        },
        "state": {"required": True, "type": "str", "choices": ["present", "absent"]},
        "system_vdom_exception": {
            "required": False,
            "type": "dict",
            "default": None,
            "options": {},
        },
    }
    for attribute_name in module_spec["options"]:
        fields["system_vdom_exception"]["options"][attribute_name] = module_spec[
            "options"
        ][attribute_name]
        if mkeyname and mkeyname == attribute_name:
            fields["system_vdom_exception"]["options"][attribute_name][
                "required"
            ] = True

    module = AnsibleModule(argument_spec=fields, supports_check_mode=True)
    check_legacy_fortiosapi(module)

    is_error = False
    has_changed = False
    result = None
    diff = None

    versions_check_result = None
    if module._socket_path:
        connection = Connection(module._socket_path)
        if "access_token" in module.params:
            connection.set_custom_option("access_token", module.params["access_token"])

        if "enable_log" in module.params:
            connection.set_custom_option("enable_log", module.params["enable_log"])
        else:
            connection.set_custom_option("enable_log", False)
        fos = FortiOSHandler(connection, module, mkeyname)
        versions_check_result = check_schema_versioning(
            fos, versioned_schema, "system_vdom_exception"
        )

        is_error, has_changed, result, diff = fortios_system(
            module.params, fos, module.check_mode
        )

    else:
        module.fail_json(**FAIL_SOCKET_MSG)

    if versions_check_result and versions_check_result["matched"] is False:
        module.warn(
            "Ansible has detected version mismatch between FortOS system and your playbook, see more details by specifying option -vvv"
        )

    if not is_error:
        if versions_check_result and versions_check_result["matched"] is False:
            module.exit_json(
                changed=has_changed,
                version_check_warning=versions_check_result,
                meta=result,
                diff=diff,
            )
        else:
            module.exit_json(changed=has_changed, meta=result, diff=diff)
    else:
        if versions_check_result and versions_check_result["matched"] is False:
            module.fail_json(
                msg="Error in repo",
                version_check_warning=versions_check_result,
                meta=result,
            )
        else:
            module.fail_json(msg="Error in repo", meta=result)


if __name__ == "__main__":
    main()
